

function SpokesMinExcErrLocGlobSARPuPow_fmincon(filepath,display,do_mls,opt_grad,freq,freq_weights)
% function SpokesMinExcErrLocGlobSARPuPow_fmincon(filepath,display,do_mls,opt_grad,freq,freq_weights)

close all;

% read parameters
disp('Reading pulse parameters ...');
[adj system opt spokes sar]=read_spokes_def_file(filepath);
if display==1
    opt.display='on';
else
    opt.display='off';
end

% read sar matrices
disp('Reading SAR matrices ...');
% sar=prepare_sar(sar,spokes,opt);

% prepare optimization
disp('Preparing optimization ...');
adj=prepare_optimization(opt,spokes,system,adj);
spokes=prepare_pulse(spokes,opt,system);

% slice profiles
fprintf('Computing slice profiles ...\n');
lz=spokes.sthick*2;
nz=300;
zs=([1:nz]'-0.5)/nz*lz - lz/2;
SP=zeros(nz,1);
[mz_x mz_y mz_z]=bloch_simulation_z(spokes,system,zs);
SP(:)=sqrt( abs(mz_x).^2 + abs(mz_y).^2 );
SP(:)=SP(:) / max(SP(:));

zind=find( abs(zs-adj.zs)<=spokes.sthick/2  );
SP_mean=mean( SP(zind),1 );

if display==1
    figure; plot(zs*1e3,SP,'-o');
    xlabel('z [mm]');
    title('Slice profile for all channels');
    axis([-lz/2*1000 lz/2*1000 0 1.1]);
end

% optimize gradient trajectory (Larmor frequency / LS cost function)
if opt_grad==1
    fprintf('Optimizing gradient trajectory ...\n');
    display_=opt.display;
    opt.display='off';  % force no display during optimization of the spokes locations           
    spokes=opt_gradients(spokes,system,opt,sar,adj,do_mls,SP_mean);
    opt.display=display_;
    spokes.spcoords
    spokes=prepare_pulse(spokes,opt,system);
end

% prepare pulse & compute SM (multiple frequencies)
disp('Preparing pulse ...');
ncunk=system.ncoils*spokes.nspokes;
nfreq=size(freq,2);
if size(freq_weights,2)~=nfreq
    error('Incorrect number of frequencies.');
end
opt.A_allfreq=zeros(nfreq*adj.nnonzeropixels,ncunk);
adj.mtt_allfreq=zeros(nfreq*adj.nnonzeropixels,1);
for i=1:nfreq
    A=compute_system_matrix(adj,system,spokes,freq(i),SP_mean);
    ind_range=(i-1)*adj.nnonzeropixels + 1 : i*adj.nnonzeropixels;
    opt.A_allfreq( ind_range,: )=A*sqrt( freq_weights(i) );  % vertical concatenation of SM for all freq
    adj.mtt_allfreq( ind_range,1 )=adj.mtt*sqrt( freq_weights(i) );  % vertical concatenation of target mag for all freq
end

% optimize RF waveforms
options=optimset('Display','iter-detailed','MaxFunEvals',10^5,'TolFun',1e-5,'TolCon',1e-5,'MaxIter',5000,...
    'Algorithm','interior-point','GradObj','on','GradConstr','on');

% BC mode
if do_mls==-1
    fprintf('******** COMPUTING BIRDCAGE MODE ********\n');
    
    % pha=[1:system.ncoils]'/system.ncoils*2*pi * 1;  % BC    
    pha=angle(squeeze(adj.b1maps(:,end/2,end/2)));  % pseudo-CP
    rf=exp(1j*pha);
    MT=opt.A_allfreq*rf;
    voltage=mean(abs(adj.mtt_allfreq))/mean(abs(MT));
    if voltage>system.umax
        voltage=system.umax;
    end
    rf=rf * voltage;
    xrf=[ real(rf);imag(rf) ];    
    
elseif do_mls==0 || do_mls==1
    
    if do_mls==0        
        fprintf('******** COMPUTING LEAST-SQUARES SOLUTION ********\n');
    else
        fprintf('******** COMPUTING MLS SOLUTION (FULL OPTIMIZATION) ********\n');
    end

    % LS
    options.Hessian='user-supplied';
    options.HessFcn=@(xrf,lambda)dualhess(xrf,lambda,opt.A_allfreq,adj,spokes,sar);
    xrf0=zeros(2*ncunk,1);
    [xrf fval exitflag output lambda]=fmincon(@(xrf)f0(xrf,opt.A_allfreq,adj),xrf0,[],[],[],[],[],[],@(xrf)fn(xrf,adj,spokes,sar,system),options);    

    % MLS refinement
    % lambda=ones(100,1);
    % [hess_an hess_numer]=test_hessian( @(xrf)f0_MLS(xrf,opt.A_allfreq,adj),@(xrf)dualhess_MLS(xrf,lambda,opt.A_allfreq,adj,spokes,sar,1),xrf );
    % [hess_an hess_numer]=test_hessian( @(xrf)fn(xrf,adj,spokes,sar,system),@(xrf)dualhess_MLS(xrf,lambda,opt.A_allfreq,adj,spokes,sar,1),xrf );
    if do_mls==1
        options.Hessian='user-supplied';
        options.HessFcn=@(xrf,lambda)dualhess_MLS(xrf,lambda,opt.A_allfreq,adj,spokes,sar,1);    
        % options.Hessian='';    
        [xrf fval exitflag output lambda]=fmincon(@(xrf)f0_MLS(xrf,opt.A_allfreq,adj),xrf,[],[],[],[],[],[],@(xrf)fn(xrf,adj,spokes,sar,system),options);
    end
    
elseif do_mls==2
    fprintf('******** COMPUTING MLS SOLUTION (PHASE ADOPTION) ********\n');
    xrf=design_pulse_mls(opt,spokes,sar,adj,system,options,freq);
end

rf=xrf(1:ncunk,1) + 1j*xrf(ncunk+1:2*ncunk,1);

% process results
display_optimization_summary(rf,adj,spokes,sar,system);
rf2=write_and_display_pulse(rf,opt,spokes,system);
MT_STA=write_and_display_magnetization(rf,opt,adj,system,spokes,freq,freq_weights);
MT_Bloch=cell(nfreq,1);

% lz=spokes.sthick;
% nz=20;
% adj_3D=extend_maps_z_direction(opt,adj,nz,lz,spokes);

for i=1:nfreq
    % [mx my mz]=bloch_simulation(rf2,adj_3D,spokes,system,freq(i));
    % MT_Bloch{i,1}=mx+1j*my );
    
    [mx my mz]=bloch_simulation(rf2,adj,spokes,system,freq(i));    
    MT_Bloch{i,1}=( mx+1j*my )*SP_mean;
end
write_and_display_bloch_simulation(MT_Bloch,opt,adj,freq);

g=spokes.g;
k=spokes.k;
SP=[zs SP];
save pulse.mat rf rf2 g k MT_STA MT_Bloch SP freq freq_weights;




function adj2=extend_maps_z_direction(opt,adj,nz,lz,spokes)

gamma=42.576*1e6;

adj2=adj;

adj2.nz=nz;
adj2.lz=lz;
adj2.zs=([1:nz]'-0.5)/nz*lz - lz/2;

% extend B1+ maps and ROI
adj2.b1maps=repmat( adj2.b1maps,[1 1 1 adj2.nz] );
adj2.roi=repmat( adj2.roi,[1 1 adj2.nz] );
adj2.b0map=repmat( adj2.b0map,[1 1 adj2.nz] );

% non-zero indices
adj2.ind=find(adj2.roi>0);
adj2.nnonzeropixels=size(adj2.ind,1);
[indx indy indz]=ind2sub( [adj2.nx adj2.ny adj2.nz],adj2.ind );
[tmpx tmpy tmpz]=ndgrid(adj2.xs,adj2.ys,adj2.zs);
adj2.nonzeropixels=[indx indy indz tmpx(adj2.ind) tmpy(adj2.ind) tmpz(adj2.ind)];









