

function xrf=design_pulse_mls(opt,spokes,sar,adj,system,options,freq)
% function xrf=design_pulse_mls(opt,spokes,sar,adj,system,options,freq)

tol_mls=1e-3;
niters=20;

global pha;
global phaimg;

ncunk=size(opt.A_allfreq,2);
iter=0;
phaimg=zeros(adj.nz,adj.nx,adj.ny);  % careful, z is the first dimension!!
smfilter=fspecial('gaussian',[10 10],3);  % 5 pixels Gaussian filter with 1 pixel s.d.

xx0=zeros(2*ncunk,1);
update_target_phase(opt.A_allfreq,ncunk,adj,xx0,smfilter,freq);
adj.mtt_allfreq=abs(adj.mtt_allfreq).*exp(1j*pha);

if strcmpi(opt.display,'on')
    f=figure;
    to_disp=conv_3Dstack_2_2Dimg(permute(phaimg,[2 3 1]),1,adj.nz);
    imagesc(to_disp); colormap(hot); colorbar; axis off; axis image; caxis([-pi pi]);
    title(sprintf('Target phase (MLS it. #%d)',iter));
    drawnow;
end

% MLS
f1=-1;
f2=-1;
options.Hessian='user-supplied';
options.HessFcn=@(xrf,lambda)dualhess(xrf,lambda,opt.A_allfreq,adj,spokes,sar);
options.Display='none';

while iter<niters || f1<0 || f2<0
    
    iter=iter+1;
    % solve LS with current phase
    xrf0=zeros(2*ncunk,1);
    [xrf f2 exitflag output lambda]=fmincon(@(xrf)f0(xrf,opt.A_allfreq,adj),xrf0,[],[],[],[],[],[],@(xrf)fn(xrf,adj,spokes,sar,system),options);  

    % iterations statement
    fprintf('\tMLS it. #%d: f=%f\n',iter,f2);
    
    % stopping criterion
    if f1>=0 && f2>=0 && ( abs(f1-f2)<(abs(f1)+abs(f2))*tol_mls || (f2>f1 && iter>1) )
        break;
    end
    
    % update target magnetization phase
    update_target_phase(opt.A_allfreq,ncunk,adj,xrf,smfilter,freq);
    adj.mtt_allfreq=abs(adj.mtt_allfreq).*exp(1j*pha);
    
    % display
    if strcmpi(opt.display,'on')
        to_disp=conv_3Dstack_2_2Dimg(permute(phaimg,[2 3 1]),1,adj.nz);
        imagesc(to_disp); colormap(hot); colorbar; axis off; axis image; caxis([-pi pi]);
        title(sprintf('Target phase (MLS it. #%d)',iter));
        drawnow;
    end
    
    f1=f2;
    
end










 

function update_target_phase(A,ncunk,adj,xx,smfilter,freq)

global pha;
global phaimg;

x=xx(1:ncunk) + 1j*xx(ncunk+1:2*ncunk);
y=A*x;
pha=angle(y);

ii = find(freq==0);
ind2 = (ii-1) * adj.nnonzeropixels +1 : ii*adj.nnonzeropixels;
phaimg(adj.ind)=pha(ind2);

















