

function rf=write_and_display_pulse(x,opt,adj,system)
% function write_and_display_pulse(x,opt,adj,system)

nchannels=system.ncoils;

file=fopen( sprintf('%s/pulse.txt',opt.resdir) ,'w');
fprintf(file,'\ntime (s)\tgx (T/m)\tgy (T/m)\tgz (T/m)\t\tsx (T/m/s)\tsy (T/m/s)\tsz (T/m/s)\tkx (1/m)\tky (1/m)\tkz (1/m)');
for j=1:nchannels
    fprintf(file,'\trf_%d_mag (V)\trf_%d_pha (deg.)',j,j);
end
time=[1:opt.ntimes]'*system.deltat;
rf=reshape(x,opt.ntimesrf,nchannels);
rf=[ zeros(opt.ntimes-opt.ntimesrf,nchannels);rf ];
for i=1:opt.ntimes
    fprintf(file,'\n');
    if i==1
        sr=[0 0 0];
    else
        sr=(opt.g(i,:)-opt.g(i-1,:))/system.deltat;
    end
	fprintf(file,'%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e\t%e',time(i),opt.g(i,:),sr,opt.k(i,:));
    for j=1:nchannels
        fprintf(file,'\t%e\t%e',abs(rf(i,j)),angle(rf(i,j))/pi*180.0);
    end
end
fclose(file);

if opt.display==1
    figure;
    subplot(2,1,1); plot(time*1e3,abs(rf)); title('RF voltage amplitudes [V]');
    subplot(2,1,2); plot(time*1e3,angle(rf)/pi*180.0); xlabel('time [ms]'); title('RF voltage phases [deg.]');
end

% write ini file
options.FACTOROVERSAMPLE=1;
rf2=rf;

% options.NOMFLIPANGLE=opt.tfa;
options.NOMFLIPANGLE=15;  % although the actual flip-angle is defined by opt.tfa, it is useful to "pretend" that the 
                           % it is always 15 degrees, which is the
                           % flip-angle in the default FLASH_ptx protocol.
                           % This avoids having to manually change the
                           % protocol flip-angle so that it matches the
                           % designed flip-angle (if there is a mismatch between the two, then
                           % the pulse is scaled by their ratio in the
                           % sequence)

options.RFPULSE_ID=0;
options.VERBOSE=0;
options.GRADRASTERTIME=system.deltat*1e6;  % us
grad=[opt.g(:,2) opt.g(:,1) opt.g(:,3)]*1e3;  % swap gx and gy and convert T/m -> mT/m
grad_over_samp_fac=1e-5/system.deltat;
if ceil(grad_over_samp_fac)~=grad_over_samp_fac
    warning('The oversampling factor is not integer, this will cause the ini pulse to be unplayable on the scanner.');
else
    options.GRADRASTERTIME=system.deltat*grad_over_samp_fac*1e6;  % 10 us
    options.FACTOROVERSAMPLE=grad_over_samp_fac;
    % make sure that rf has a size that's dividable by grad_over_samp_fac
    while mod(size(rf2,1),grad_over_samp_fac)~=0
        rf2=[ rf2;zeros(1,size(rf2,2)) ];
        grad=[ grad;zeros(1,size(grad,2)) ];
    end
    x_s=linspace(0,1,size(grad,1))';
    x_t=linspace(0,1,size(grad,1)/grad_over_samp_fac)';
    tmpx=interp1(x_s,grad(:,1),x_t);  % interpolate gx
    tmpy=interp1(x_s,grad(:,2),x_t);  % interpolate gy
    tmpz=interp1(x_s,grad(:,3),x_t);  % interpolate gz
    grad=[tmpx tmpy tmpz];
end
save_pTXRFPulse_toINI(grad,reshape(rf2,system.ncoils*size(rf2,1),1),[],options);
    
    

    
    
    
    
    
    
    
    
    
    
    
